const fs = require('fs')
const taskList = [];
const failedTasks =[]
let maxNumberOfRunnigTasks = 20;
let numberOfRunnigTasks = 0;

let successTaskCount = 0
let failedTaskCount = 0
let allTasksCount = 0

function addTask(task) {
    console.log('New task added ', task.name)
    allTasksCount++
    taskList.push(task);
    handleTask()
}

function taskCompleted(task, result) {
    console.log('task completed : ', task.name)
    successTaskCount++
    numberOfRunnigTasks--;
    handleTask()

}

function taskFailed(task, error) {
    console.log('task failed : ', {taskName: task.name, errorMessage: error.message})
    failedTasks.push({taskName: task.name,
        errorMessage: error.message,
        errorResponse : error && error.response && error.response.data})
    failedTaskCount++
    numberOfRunnigTasks--;
    if (task.maxRetries > task.retries) {
        task.retries++;
        addTask(task)
        return
    }
    handleTask()
}

function execute(task) {
    console.log('Execute new task ',task && task.name)

    if (!task) {
        //logically this statement should be unreachable, but some times code come here
        console.error("task is undefined")
        return
    }
    task.executor().then((result) => {
        taskCompleted(task, result)
    }).catch((error) => {
        taskFailed(task, error)
    })
}

function canRunTask() {
    return maxNumberOfRunnigTasks > numberOfRunnigTasks;
}

function handleTask() {
    try {
        if (canRunTask() && taskList.length == 0 && (successTaskCount + failedTaskCount) === allTasksCount) {
            console.log('No more tasks')
            console.log("report summary", {
                allTasksCount,
                failedTaskCount,
                successTaskCount
            })
            const failedTaskDir = "./failedTasks.txt"
            fs.writeFileSync(failedTaskDir, JSON.stringify({failedTasks}, null ,4))
            console.log("errrors saved in ", failedTaskDir)
            console.log("Run this command to update tour movie list\nimdb index")
            process.exit()
            return
        } else if (canRunTask()) {
            numberOfRunnigTasks++;
            let currentTask = taskList.shift();
            execute(currentTask)
        }
    } catch (e) {
        console.log("handleTask error ", e)
    }


}

module.exports = {
    addTask
};


