let dataService = require('./../services/dataService');
const fs = require('fs');
const kue = require('../kueHandler/kue');
const stringSimilarity = require('string-similarity');


/**
 * Created by renjer on 3/30/18.
 */

/**
 * we must rename movie directory after getting info, so this function should
 * for example if thi function get inception info should return
 * inception (2010) -* 8.8
 * @param movieInfo
 */
function getMovieDirectoryNameByInfo(movieInfo) {
    return movieInfo.director + " -* " + movieInfo.title + " (" + movieInfo.year + ")" + " -* " + movieInfo.rating
}

/***
 * for example if name of directory is inception [2010] nolan, this function will return inception
 * @param movieDirectoryName
 */
function extractTitleFromMovieDirectoryName(movieDirectoryName) {
    const movieParts = movieDirectoryName.split(" -*")
    let titleWithYear
    if (movieParts.length === 1) {
        //example :Marriage Story (2019)
        titleWithYear = movieParts[0]
    }
    if (movieParts.length === 2) {
        //example :  Marriage Story (2019) -* 8.0
        titleWithYear = movieParts[0]
    }
    if (movieParts.length === 3) {
        //example : Noah Baumbach -* Marriage Story (2019) -* 8.0
        titleWithYear = movieParts[1]
    }
    const year = getYearFromMovieDirectoryName(movieDirectoryName)
    let title = titleWithYear
    if (year) {
        title = titleWithYear.replace(" (" + year + ")", '')
    }
    return title

}

async function fetchAndSaveMovieInfo(movie, dir, year) {
    try {
        let movieName = extractTitleFromMovieDirectoryName(movie).trim();
        console.log("movie name   : ", {movieName, year});
        const movieInfo = await dataService.getMovieInfoFromOurServers(movieName, year)
        const similarity = stringSimilarity.compareTwoStrings(movieName.toLowerCase() , movieInfo.title.toLowerCase())
        if (similarity < 0.75) {
            throw new Error('movieName and title of fetched movie doesnt match|' +
                movieName + "|   vs   |" + movieInfo.title+"|", {similarity} );
        }

        let movieDirName = getMovieDirectoryNameByInfo(movieInfo);
        fs.renameSync(dir + "/" + movie, dir + "/" + movieDirName);
        return dataService.makeFileAndPutSthInIt(movieInfo, dir + "/" + movieDirName)
    } catch (e) {
        console.log("fetchAndSaveMovieInfo error", e)
        throw e
    }

}

/***
 * get item(movie) of in directory and forEach on them and call  fetchDataSaveFilesAndRenameDirectoryOfMovie()
 */
function getDirectoryNameAndFetchMoviesItem(dir,
                                            overwriteMoviesWithDirectorName,
                                            overwriteMoviesWithRateAndWithoutDirectorName) {
    console.log("get Directory name ... called  : ");
    let moviesArray = fs.readdirSync(dir);
    moviesArray.forEach((movie) => {
        try {
            console.log("movie :  : ", movie);
            const countImdbFetchSigns = (movie.match(/-\*/g) || []).length;

            if (!fs.lstatSync(`${dir}/${movie}`).isDirectory()) {
                console.log(`${movie} is not directory`)
                return
            }
            if (!overwriteMoviesWithDirectorName && !overwriteMoviesWithRateAndWithoutDirectorName && countImdbFetchSigns > 0) {
                console.log("movie info saved before  : ", movie);
                return;
            }
            if (!overwriteMoviesWithDirectorName && overwriteMoviesWithRateAndWithoutDirectorName && countImdbFetchSigns > 1) {
                console.log("movie info saved before  : ", movie);
                return;
            }


            const year = getYearFromMovieDirectoryName(movie)
            // fetchAndSaveMovieInfo(movie, dir);
            let task = {
                taskTyp: "movieInfo",
                name: movie,
                executor: fetchAndSaveMovieInfo.bind(null, movie, dir, year | undefined),
                maxRetries: 2
            };
            kue.addTask(task);
        } catch (e) {
        }

    });

}

/**
 * input ="Marriage Story (2019) -* 8.0"
 * output ="2019"
 * @param movieDirName
 * @returns {{groups: {}}|RegExpExecArray|string}
 */
function getYearFromMovieDirectoryName(movieDirName) {
    const match = /\((\d{4})\)/.exec(movieDirName)
    return match && match[1]
}

/***
 * movie item is name of directory of movie, that this function do all thing for movie
 * @param movieItem
 */
function fetchDataSaveFilesAndRenameDirectoryOfMovie(movieItem) {

}


function indexMoviesInfo(dir) {
    const moviesDir = __dirname + "/../web/src/data/movies.json";
    let movies = [];
    let items = fs.readdirSync(dir);
    try {
        let newMovies = require(moviesDir)
        movies.push(...newMovies)
    } catch (e) {
        console.log("exception of indexing movies")
    }
    items.forEach(item => {
        try {
            const infoPath = dir + "/" + item + "/info.txt";
            let fileExist = fs.existsSync(infoPath);
            // if (item.includes("-*")){
            if (fileExist) {
                const movieStr = fs.readFileSync(infoPath);
                let movie = JSON.parse(movieStr)
                const isMovieExistsInMoviesInfo = movies.find((movieInfo) => {
                    return (movieInfo.title === movie.title && movieInfo.year === movie.year)
                })
                if (isMovieExistsInMoviesInfo) {
                    console.log("Movie existed ", item)
                } else {
                    movies.push(movie)
                    console.log("Movie info added : ", item)
                }

            }
        } catch (e) {
            console.log("Add movie info failed, error", e)
        }
    });

    console.log("movies dir : ", moviesDir)
    fs.writeFileSync(moviesDir, JSON.stringify(movies));
    console.log("Movies synced successfully")
    process.exit()
}

function deleteMoviesData() {
    fs.unlinkSync(__dirname + "/../web/src/data/movies.json")
    process.exit()

}

module.exports = {
    getMovieDirectoryNameByInfo,
    getDirectoryNameAndFetchMoviesItem,
    indexMoviesInfo,
    deleteMoviesData
};